POTENTIAL = {"H":"GTH-BLYP-q1", "He":"GTH-BLYP-q2", "Li":"GTH-BLYP-q3", "Be":"GTH-BLYP-q4", "B":"GTH-BLYP-q3", "C":"GTH-BLYP-q4", "N":"GTH-BLYP-q5", "O":"GTH-BLYP-q6", "F":"GTH-BLYP-q7", "Ne":"GTH-BLYP-q8", "Na":"GTH-BLYP-q1", "Mg":"GTH-BLYP-q10", "Al":"GTH-BLYP-q3", "Si":"GTH-BLYP-q4", "P":"GTH-BLYP-q5", "S":"GTH-BLYP-q6", "Cl":"GTH-BLYP-q7", "Ar":"GTH-BLYP-q8", "K":"GTH-BLYP-q9", "Ca":"GTH-BLYP-q10", "Sc":"GTH-BLYP-q11", "Ti":"GTH-BLYP-q12", "V":"GTH-BLYP-q13", "Cr":"GTH-BLYP-q14", "Mn":"GTH-BLYP-q15", "Fe":"GTH-BLYP-q16", "Co":"GTH-BLYP-q17", "Ni":"GTH-BLYP-q18", "Cu":"GTH-BLYP-q11", "Zn":"GTH-BLYP-q12", "Ga":"GTH-BLYP-q13", "Ge":"GTH-BLYP-q4", "As":"GTH-BLYP-q5", "Se":"GTH-BLYP-q6", "Br":"GTH-BLYP-q7", "Kr":"GTH-BLYP-q8", "Rb":"GTH-BLYP-q9", "Sr":"GTH-BLYP-q10", "Y":"GTH-BLYP-q11", "Zr":"GTH-BLYP-q12", "Mo":"GTH-BLYP-q14", "Ru":"GTH-BLYP-q16", "Rh":"GTH-BLYP-q17", "Pd":"GTH-BLYP-q18", "Ag":"GTH-BLYP-q11", "In":"GTH-BLYP-q13", "Sb":"GTH-BLYP-q5", "Te":"GTH-BLYP-q6", "I":"GTH-BLYP-q7", "Xe":"GTH-BLYP-q8", "Cs":"GTH-BLYP-q9", "Ba":"GTH-BLYP-q10", "Ce":"GTH-BLYP-q12", "Gd":"GTH-BLYP-q18", "W":"GTH-BLYP-q14", "Au":"GTH-BLYP-q11", "Pb":"GTH-BLYP-q4", "Bi":"GTH-BLYP-q15"}
BASIS = {"H":"DZVP-MOLOPT-SR-GTH-q1", "He":"DZVP-MOLOPT-SR-GTH-q2", "Li":"DZVP-MOLOPT-SR-GTH-q3", "Be":"DZVP-MOLOPT-SR-GTH-q4", "B":"DZVP-MOLOPT-SR-GTH-q3", "C":"DZVP-MOLOPT-SR-GTH-q4", "N":"DZVP-MOLOPT-SR-GTH-q5", "O":"DZVP-MOLOPT-SR-GTH-q6", "F":"DZVP-MOLOPT-SR-GTH-q7", "Ne":"DZVP-MOLOPT-SR-GTH-q8", "Na":"DZVP-MOLOPT-SR-GTH-q9", "Mg":"DZVP-MOLOPT-SR-GTH-q10", "Al":"DZVP-MOLOPT-SR-GTH-q3", "Si":"DZVP-MOLOPT-SR-GTH-q4", "P":"DZVP-MOLOPT-SR-GTH-q5", "S":"DZVP-MOLOPT-SR-GTH-q6", "Cl":"DZVP-MOLOPT-SR-GTH-q7", "Ar":"DZVP-MOLOPT-SR-GTH-q8", "K":"DZVP-MOLOPT-SR-GTH-q9", "Ca":"DZVP-MOLOPT-SR-GTH-q10", "Sc":"DZVP-MOLOPT-SR-GTH-q11", "Ti":"DZVP-MOLOPT-SR-GTH-q12", "V":"DZVP-MOLOPT-SR-GTH-q13", "Cr":"DZVP-MOLOPT-SR-GTH-q14", "Mn":"DZVP-MOLOPT-SR-GTH-q15", "Fe":"DZVP-MOLOPT-SR-GTH-q16", "Co":"DZVP-MOLOPT-SR-GTH-q17", "Ni":"DZVP-MOLOPT-SR-GTH-q18", "Cu":"DZVP-MOLOPT-SR-GTH-q11", "Zn":"DZVP-MOLOPT-SR-GTH-q12", "Ga":"DZVP-MOLOPT-SR-GTH-q13", "Ge":"DZVP-MOLOPT-SR-GTH-q4", "As":"DZVP-MOLOPT-SR-GTH-q5", "Se":"DZVP-MOLOPT-SR-GTH-q6", "Br":"DZVP-MOLOPT-SR-GTH-q7", "Kr":"DZVP-MOLOPT-SR-GTH-q8", "Rb":"DZVP-MOLOPT-SR-GTH-q9", "Sr":"DZVP-MOLOPT-SR-GTH-q10", "Y":"DZVP-MOLOPT-SR-GTH-q11", "Zr":"DZVP-MOLOPT-SR-GTH-q12", "Nb":"DZVP-MOLOPT-SR-GTH-q13", "Mo":"DZVP-MOLOPT-SR-GTH-q14", "Tc":"DZVP-MOLOPT-SR-GTH-q15", "Ru":"DZVP-MOLOPT-SR-GTH-q16", "Rh":"DZVP-MOLOPT-SR-GTH-q17", "Pd":"DZVP-MOLOPT-SR-GTH-q18", "Ag":"DZVP-MOLOPT-SR-GTH-q11", "Cd":"DZVP-MOLOPT-SR-GTH-q12", "In":"DZVP-MOLOPT-SR-GTH-q13", "Sn":"DZVP-MOLOPT-SR-GTH-q4", "Sb":"DZVP-MOLOPT-SR-GTH-q5", "Te":"DZVP-MOLOPT-SR-GTH-q6", "I":"DZVP-MOLOPT-SR-GTH-q7", "Xe":"DZVP-MOLOPT-SR-GTH-q8", "Cs":"DZVP-MOLOPT-SR-GTH-q9", "Ba":"DZVP-MOLOPT-SR-GTH-q10", "Hf":"DZVP-MOLOPT-SR-GTH-q12", "Ta":"DZVP-MOLOPT-SR-GTH-q13", "W":"DZVP-MOLOPT-SR-GTH-q14", "Re":"DZVP-MOLOPT-SR-GTH-q15", "Os":"DZVP-MOLOPT-SR-GTH-q16", "Ir":"DZVP-MOLOPT-SR-GTH-q17", "Pt":"DZVP-MOLOPT-SR-GTH-q18", "Au":"DZVP-MOLOPT-SR-GTH-q11", "Hg":"DZVP-MOLOPT-SR-GTH-q12", "Tl":"DZVP-MOLOPT-SR-GTH-q13", "Pb":"DZVP-MOLOPT-SR-GTH-q4", "Bi":"DZVP-MOLOPT-SR-GTH-q5", "Po":"DZVP-MOLOPT-SR-GTH-q6", "At":"DZVP-MOLOPT-SR-GTH-q7", "Rn":"DZVP-MOLOPT-SR-GTH-q8"}
